import { Injectable } from '@angular/core'
import {
    HttpRequest,
    HttpHandler,
    HttpEvent,
    HttpInterceptor,
    HttpResponse,
    HttpErrorResponse
} from '@angular/common/http'
import { Observable, throwError } from 'rxjs'
import { catchError, map } from 'rxjs/operators'
import { LoadingService } from '../services/loading.service'
import { AuthService } from '../services/auth.service'
import { NzMessageService } from 'ng-zorro-antd/message'

/**
 * This class is for intercepting http requests. When a request starts, we set the loadingSub property
 * in the LoadingService to true. Once the request completes and we have a response, set the loadingSub
 * property to false. If an error occurs while servicing the request, set the loadingSub property to false.
 * @class {HttpRequestInterceptor}
 */
@Injectable({
    providedIn: 'root'
})
export class HttpRequestInterceptor implements HttpInterceptor {
    constructor(
        private _loading: LoadingService,
        private _messageService: NzMessageService,
        private _authService: AuthService
    ) {}

    intercept(request: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {
        if (!request.url.includes('auth/token/refresh')) this._loading.setLoading(true, request.url)
        return next
            .handle(request)
            .pipe(
                catchError((err: HttpErrorResponse) => {
                    console.log('error', err)
                    if (err.status === 0)
                        this._messageService.error(
                            $localize`:@@toastMessageCouldConnectBackend:Could'nt connect to backend`
                        )
                    else if (err.error.code === 'user_not_found') {
                        this._messageService.error(
                            $localize`:@@toastMessageAccountNotFound: Account not found`
                        )
                        this._authService.logout()
                    } else
                        this._messageService.error(
                            err.error?.message ?? err.statusText
                        )
                    this._loading.setLoading(false, request.url)
                    return throwError(() => Error(`${err.status} ${err.message}`))
                })
            )
            .pipe(
                map<HttpEvent<any>, any>((evt: HttpEvent<any>) => {
                    if (evt instanceof HttpResponse) {
                        this._loading.setLoading(false, request.url)
                    }
                    return evt
                })
            )
    }
}
